//******************************************************************************
//******************************************************************************
//*   Copyright (C) 2010 K. Vasudevan. Indian Institute of Technology Kanpur.
//*   All Rights Reserved.
//******************************************************************************
//  This code uses only prefix. Genie aided channel estimation, that is, the
//  channel is estimated assuming data symbols are known. Next, the data
//  symbols are detected using the estimated channel coefficients.
//  Synchronization is assumed. Uncoded OFDM.
//  Code tested in SCILAB ver 5.3.3.
//******************************************************************************
clear;
timer();
//******************************************************************************
// Constants.
//******************************************************************************
MAX_FRAMES=1;                         // No. of OFDM symbols to be simulated.
OFDM_SIZE=512;            // Length of OFDM symbol, excluding the cyclic prefix.
CHAN_MEM=9;                                         // Length of channel memory.
CHAN_LEN=CHAN_MEM+1;                                       // Length of channel.
EST_CHAN_MEM=2*CHAN_MEM;                     // Estimated memory of the channel.
EST_CHAN_LEN=EST_CHAN_MEM+1;                        // Estimated channel length.
SNR_DB=20.0;                                                       // SNR in dB.
FADE_MEAN=0.0;                                                // Mean of fading.
FADE_VAR=0.5;                               // Variance per dimension of fading.
FADE_STD=sqrt(FADE_VAR);                            // Std. deviation of fading.
NOISE_MEAN=0.0;                                                 // Mean of AWGN.
NOISE_VAR=CHAN_LEN/(OFDM_SIZE*10^(0.1*SNR_DB));    // Var per dimension of AWGN.
NOISE_STD=sqrt(NOISE_VAR);                           // Std. deviation of noise.
QPSK_SYM1=(+1+%i)*ones(1,OFDM_SIZE);                  // QPSK symbols of type 1.
QPSK_SYM2=(-1+%i)*ones(1,OFDM_SIZE);                  // QPSK symbols of type 2.
QPSK_SYM3=(-1-%i)*ones(1,OFDM_SIZE);                  // QPSK symbols of type 3.
QPSK_SYM4=(+1-%i)*ones(1,OFDM_SIZE);                  // QPSK symbols of type 4.
//******************************************************************************
Chan_Ip_Matrix=zeros(OFDM_SIZE,EST_CHAN_LEN);           // Channel input matrix.
//******************************************************************************
Ber=0;
for Frame_Cnt=1:MAX_FRAMES
//******************************************************************************
// Generate QPSK symbols.
//******************************************************************************
 Re_Tx_Bits=grand(1,OFDM_SIZE,'def');
 Im_Tx_Bits=grand(1,OFDM_SIZE,'def');
 Re_Tx_Bits=int(2*Re_Tx_Bits);
 Im_Tx_Bits=int(2*Im_Tx_Bits);
 Re_Tx_Sym=1.0-2.0*Re_Tx_Bits;
 Im_Tx_Sym=1.0-2.0*Im_Tx_Bits;
 Tx_Sym=Re_Tx_Sym+%i*Im_Tx_Sym;
//******************************************************************************
 Sym_IFFT=fft(Tx_Sym.',1).';            // IFFT. There is division by OFDM_SIZE.
 Sym_IFFT_Energy=Sym_IFFT*Sym_IFFT';           // Energy of transmitted samples.
//******************************************************************************
// Add cyclic prefix.
//******************************************************************************
 Chan_Ip=[Sym_IFFT(OFDM_SIZE-EST_CHAN_MEM+1:OFDM_SIZE) Sym_IFFT];
//******************************************************************************
// Get the channel impulse response.
//******************************************************************************
Re_Chan=zeros(1,CHAN_LEN);           // Initialize re. channel impulse response.
Im_Chan=zeros(1,CHAN_LEN);           // Initialize im. channel impulse response.
Re_Chan=grand(Re_Chan,'nor',FADE_MEAN,FADE_STD);                 // Re. channel.
Im_Chan=grand(Im_Chan,'nor',FADE_MEAN,FADE_STD);                 // Im. channel.
Chan=Re_Chan+%i*Im_Chan;                                     // Complex channel.
Chan_Ext_Zeros=[Chan zeros(1,OFDM_SIZE-CHAN_LEN)];              // Zero padding.
Chan_FFT=fft(Chan_Ext_Zeros.',-1).';                     // OFDM_SIZE-point FFT.
//******************************************************************************
// Get channel input matrix and channel estimation matrix.
//******************************************************************************
 for i=1:OFDM_SIZE
  Chan_Ip_Matrix(i,:)=Chan_Ip(EST_CHAN_LEN+i-1:-1:i);
 end
 Chan_Est_Matrix=inv(Chan_Ip_Matrix'*Chan_Ip_Matrix)*Chan_Ip_Matrix';
//******************************************************************************
 MF_Coeff=conj([Sym_IFFT(OFDM_SIZE:-1:1)]);
 Chan_Op=convol(Chan,Chan_Ip);
 Re_Noise=grand(Chan_Op,'nor',NOISE_MEAN,NOISE_STD);
 Im_Noise=grand(Chan_Op,'nor',NOISE_MEAN,NOISE_STD);
 Noise=Re_Noise+%i*Im_Noise;
 Chan_Op=Chan_Op+Noise;
 Sync_Op=convol(MF_Coeff,Chan_Op)/Sym_IFFT_Energy;
// subplot(121);
// plot(abs(Sync_Op));
 Rx_Sig=Chan_Op(EST_CHAN_LEN:OFDM_SIZE+EST_CHAN_MEM);   // Steady-state samples.
 Chan_Est=Chan_Est_Matrix*Rx_Sig.';
 Chan_Est_Ext_Zeros=[Chan_Est;zeros(OFDM_SIZE-EST_CHAN_LEN,1)];
 Est_Chan_FFT=fft(Chan_Est_Ext_Zeros,-1).';
// subplot(122);
// plot(abs(Chan_Est));
 Rx_FFT_Op=fft(Rx_Sig.',-1).';
// Rx_Sym=Rx_FFT_Op./Chan_FFT;
 Dist1=abs(Rx_FFT_Op-QPSK_SYM1.*Est_Chan_FFT);           // Sq. dist. wrt sym 1.
 Dist2=abs(Rx_FFT_Op-QPSK_SYM2.*Est_Chan_FFT);           // Sq. dist. wrt sym 2.
 Dist3=abs(Rx_FFT_Op-QPSK_SYM3.*Est_Chan_FFT);           // Sq. dist. wrt sym 3.
 Dist4=abs(Rx_FFT_Op-QPSK_SYM4.*Est_Chan_FFT);           // Sq. dist. wrt sym 4.
 [M,K]=min(Dist1,Dist2,Dist3,Dist4);               // Get min. dist and indices.
 Rx_Sym=round(exp(%i*(K*%pi/2-%pi/4)));                    // Estimated symbols.
// Rx_Sym=round(Rx_Sym);
 Ber=Ber+nnz(Tx_Sym-Rx_Sym);
end
//******************************************************************************
timer()
//******************************************************************************
