//******************************************************************************
//******************************************************************************
//*   Copyright (C) 2016 K. Vasudevan. Indian Institute of Technology Kanpur.
//*   All Rights Reserved.
//******************************************************************************
// This code computes the average error rates (coded BER, uncoded BER and
// probability of erasure). Note that the "*_avg.txt" files have only one
// line containing the average value.
// Code tested on SCILAB ver 5.5.2.
//******************************************************************************
// Open files for reading.
//******************************************************************************
SNR_DB=0.0;                                               // SNR per bit in dB.
x=string(SNR_DB);
y = strsubst(x, '.', '_');
fd_u_ber_w = mopen(pwd()+'/u_ber_'+y+'.txt', 'rt');         // Uncoded BER file.
fd_c_ber_w = mopen(pwd()+'/c_ber_'+y+'.txt', 'rt');           // Coded BER file.
fd_f_eer_w = mopen(pwd()+'/f_eer_'+y+'.txt', 'rt');             // Erasure file.
fd_rf1_w = mopen(pwd()+'/rf1_'+y+'.txt', 'rt');              // RMS FOFF 1 file.
fd_rf2_w = mopen(pwd()+'/rf2_'+y+'.txt', 'rt');              // RMS FOFF 2 file.
fd_rf3_w = mopen(pwd()+'/rf3_'+y+'.txt', 'rt');              // RMS FOFF 3 file.
fd_mf1_w = mopen(pwd()+'/mf1_'+y+'.txt', 'rt');              // Max FOFF 1 file.
fd_mf2_w = mopen(pwd()+'/mf2_'+y+'.txt', 'rt');              // Max FOFF 2 file.
fd_mf3_w = mopen(pwd()+'/mf3_'+y+'.txt', 'rt');              // Max FOFF 3 file.
//******************************************************************************
// Read the files.
//******************************************************************************
u_ber_w_mat=mfscanf(-1,fd_u_ber_w,'%f %f');
c_ber_w_mat=mfscanf(-1,fd_c_ber_w,'%f %f');
f_eer_w_mat=mfscanf(-1,fd_f_eer_w,'%f %f');
rf1_w_mat=mfscanf(-1,fd_rf1_w,'%f %f');
rf2_w_mat=mfscanf(-1,fd_rf2_w,'%f %f');
rf3_w_mat=mfscanf(-1,fd_rf3_w,'%f %f');
mf1_w_mat=mfscanf(-1,fd_mf1_w,'%f %f');
mf2_w_mat=mfscanf(-1,fd_mf2_w,'%f %f');
mf3_w_mat=mfscanf(-1,fd_mf3_w,'%f %f');
//******************************************************************************
// Open files for writing.
//******************************************************************************
fd_u_ber_avg=mopen(pwd()+'/u_ber_'+y+'_avg.txt','wt'); // Avg. uncoded BER file.
fd_c_ber_avg=mopen(pwd()+'/c_ber_'+y+'_avg.txt','wt');   // Avg. coded BER file.
fd_f_eer_avg=mopen(pwd()+'/f_eer_'+y+'_avg.txt','wt');     // Avg. erasure file.
fd_rf1_avg=mopen(pwd()+'/rf1_'+y+'_avg.txt','wt');      // Avg. RMS FOFF 1 file.
fd_rf2_avg=mopen(pwd()+'/rf2_'+y+'_avg.txt','wt');      // Avg. RMS FOFF 2 file.
fd_rf3_avg=mopen(pwd()+'/rf3_'+y+'_avg.txt','wt');      // Avg. RMS FOFF 3 file.
fd_mf1_max=mopen(pwd()+'/mf1_'+y+'_max.txt','wt');          // Max. FOFF 1 file.
fd_mf2_max=mopen(pwd()+'/mf2_'+y+'_max.txt','wt');          // Max. FOFF 2 file.
fd_mf3_max=mopen(pwd()+'/mf3_'+y+'_max.txt','wt');          // Max. FOFF 3 file.
//******************************************************************************
// Compute the average error rates.
//******************************************************************************
u_ber_avg=mean(u_ber_w_mat,'r');                            // Avg. uncoded BER.
c_ber_avg=mean(c_ber_w_mat,'r');                              // Avg. coded BER.
f_eer_avg=mean(f_eer_w_mat,'r');                                // Avg. erasure.
rf1_avg=mean(rf1_w_mat.^2,'r').^0.5;                         // Avg. RMS FOFF 1.
rf2_avg=mean(rf2_w_mat.^2,'r').^0.5;                         // Avg. RMS FOFF 2.
rf3_avg=mean(rf3_w_mat.^2,'r').^0.5;                         // Avg. RMS FOFF 3.
mf1_max=max(mf1_w_mat,'r');                                      // Max. FOFF 1.
mf2_max=max(mf2_w_mat,'r');                                      // Max. FOFF 2.
mf3_max=max(mf3_w_mat,'r');                                      // Max. FOFF 3.
//******************************************************************************
// Write the average values into the files.
//******************************************************************************
mfprintf(fd_u_ber_avg,'%f %.10f\n',SNR_DB,u_ber_avg(1,2));
mfprintf(fd_c_ber_avg,'%f %.10f\n',SNR_DB,c_ber_avg(1,2));
mfprintf(fd_f_eer_avg,'%f %.10f\n',SNR_DB,f_eer_avg(1,2));
mfprintf(fd_rf1_avg,'%f %.10f\n',SNR_DB,rf1_avg(1,2));
mfprintf(fd_rf2_avg,'%f %.10f\n',SNR_DB,rf2_avg(1,2));
mfprintf(fd_rf3_avg,'%f %.10f\n',SNR_DB,rf3_avg(1,2));
mfprintf(fd_mf1_max,'%f %.10f\n',SNR_DB,mf1_max(1,2));
mfprintf(fd_mf2_max,'%f %.10f\n',SNR_DB,mf2_max(1,2));
mfprintf(fd_mf3_max,'%f %.10f\n',SNR_DB,mf3_max(1,2));
//******************************************************************************
// Close the files.
//******************************************************************************
mclose(fd_u_ber_w);
mclose(fd_c_ber_w);
mclose(fd_f_eer_w);
mclose(fd_u_ber_avg);
mclose(fd_c_ber_avg);
mclose(fd_f_eer_avg);
mclose(fd_rf1_avg);
mclose(fd_rf2_avg);
mclose(fd_rf3_avg);
mclose(fd_mf1_max);
mclose(fd_mf2_max);
mclose(fd_mf3_max);
//******************************************************************************
